!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! XNetSend.f90 - Main application module for XNetSend sample. 
! 
! XTabEditor is a simple text-editor with tab-like interface. XTabEditor
! sample demonstrates 
! * usage of tab-controls, and edit-boxes
! * working with multi-line edit boxes and persistence of their contents
! * working with unlimited number of windows using XGetWindowData/
!   XSetWindowData
! * printing
!
! 2005. Xeffort.com
!        Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
!XInit
!XW_FRAME_OnNotifyIcon
!XW_FRAME_OnOpen
!XW_FRAME_OnClose
!XW_FRAME_OnOKCancel
!XW_FRAME_NetSend
!XW_FRAME_OnBrowse
!XW_FRAME_OnAddressBook
!UpdateList
!XAddDlg_OnAddressChange
!XAddDlg_OnGroup
!xGrDlg_OnOK
!xGrDlg_OnMemberChange
!XGrDlg_UpdateCombo
!GetGroupByName
!SaveAddressBook

MODULE XNetSendM

USE Xeffort
USE DFWIN, Nil=>NULL

IMPLICIT NONE

INTERFACE
      INTEGER FUNCTION NetMessageBufferSend(szServer, szMsg, szFrom, szText, ilenText)
      !DEC$ATTRIBUTES STDCALL, ALIAS: "_NetMessageBufferSend@20"::   NetMessageBufferSend
      !DEC$ATTRIBUTES REFERENCE, ALLOW_NULL::   szServer
      !DEC$ATTRIBUTES REFERENCE::   szMsg
      !DEC$ATTRIBUTES REFERENCE, ALLOW_NULL::   szFrom
      !DEC$ATTRIBUTES REFERENCE::   szText
      INTEGER(2)::     szServer(*)
      INTEGER(2)::     szMsg(*)
      INTEGER(2)::     szFrom(*)
      INTEGER(2)::     szText(*)
      INTEGER::        ilenText
      END FUNCTION
END INTERFACE

INTERFACE
      INTEGER FUNCTION NetMessageNameAdd(szServer, szMsg)
      !DEC$ATTRIBUTES STDCALL, ALIAS: "_NetMessageNameAdd@8"::   NetMessageNameAdd
      !DEC$ATTRIBUTES REFERENCE::   szServer
      !DEC$ATTRIBUTES REFERENCE::   szMsg
      CHARACTER(*)::   szServer
      CHARACTER(*)::   szMsg
      END FUNCTION
END INTERFACE

INTERFACE
      INTEGER FUNCTION NetMessageNameEnum(szServer, iLevel, lpBuf, iPrefMaxLen, nRead, nTotal, iHandle)
      !DEC$ATTRIBUTES STDCALL, ALIAS: "_NetMessageNameEnum@28"::   NetMessageNameEnum
      !DEC$ATTRIBUTES REFERENCE::   szServer
      !DEC$ATTRIBUTES REFERENCE::   nRead
      !DEC$ATTRIBUTES REFERENCE::   nTotal
      !DEC$ATTRIBUTES REFERENCE::   iHandle
      CHARACTER(*)::   szServer
      INTEGER::        iLevel
      INTEGER::        lpBuf
      INTEGER::        iPrefMaxLen
      INTEGER::        nRead
      INTEGER::        nTotal
      INTEGER::        iHandle
      END FUNCTION
END INTERFACE

INTERFACE
      INTEGER(4) FUNCTION  LoadImageA(hInst,lpszName,uType,cxDesired,    &
                                     cyDesired,fuLoad)
      !DEC$ATTRIBUTES STDCALL, ALIAS : '_LoadImageA@24' :: LoadImageA
      INTEGER                 hInst
      INTEGER                 lpszName
      INTEGER                 uType
      INTEGER                 cxDesired
      INTEGER                 cyDesired
      INTEGER                 fuLoad
      END FUNCTION LoadImageA
END INTERFACE

TYPE T_MSG_INFO_0
   CHARACTER(32),POINTER::  msgi0_name
END TYPE T_MSG_INFO_0

!DEC$IF (_DF_VERSION_ < 650)
type T_NOTIFYICONDATA  ! nid 
    integer(4) cbSize 
    integer(4) hWnd 
    integer(4) uID 
    integer(4) uFlags 
    integer(4) uCallbackMessage 
    integer(4) hIcon 
    character(64)  szTip
end type T_NOTIFYICONDATA

INTEGER, PARAMETER:: NIM_ADD         = #00000000
INTEGER, PARAMETER:: NIM_MODIFY      = #00000001
INTEGER, PARAMETER:: NIM_DELETE      = #00000002

INTEGER, PARAMETER:: NIF_MESSAGE     = #00000001
INTEGER, PARAMETER:: NIF_ICON        = #00000002
INTEGER, PARAMETER:: NIF_TIP         = #00000004

INTERFACE
      LOGICAL FUNCTION Shell_NotifyIcon(dwMessage, pNID)
      !DEC$ATTRIBUTES STDCALL, ALIAS: "_Shell_NotifyIcon@8"::   Shell_NotifyIcon
      INTEGER(4) dwMessage
      INTEGER(4) pNID
      END FUNCTION
END INTERFACE
!DEC$ENDIF

INTEGER, PARAMETER:: WM_ICONCLICKED = #8050

INTEGER, PARAMETER:: ID_NOTIFY_OPEN = 3
INTEGER, PARAMETER:: ID_NOTIFY_CLOSE = 4

TYPE T_XNSGROUP
   TYPE (T_XNSGROUP), POINTER::  tNext=>NULL()
   CHARACTER(32)::               sName
   INTEGER::                     nMembers
   CHARACTER(32),POINTER::       sMember(:)=>NULL()
END TYPE T_XNSGROUP

CHARACTER(256)::     sBook
TYPE(T_XNSGROUP), POINTER::      tHead
!CHARACTER(64)::                  sServer(0:4)
!INTEGER::                        nMRU = 0,  &
INTEGER::                        nGroups
LOGICAL::                        bAutoAdd = .TRUE.

INTEGER, PARAMETER::             MAX_TEXTLEN = 500
INTEGER::            hicoSmall, hicoBig, hicoBrowse, hicoAddress
CHARACTER*(*),PARAMETER::  APP_REGKEY = "Software\\Xeffort\\XNetSend"C

CONTAINS
!=====================================================================
!PURPOSE: Initialization function called by Xeffort library on app start.
INTEGER FUNCTION XInit(szCmdLine,nCmdShow)
!DEC$IF (_DF_VERSION_.GE.650)
!DEC$ATTRIBUTES DEFAULT, DECORATE, ALIAS: 'XINIT':: XInit
!DEC$ELSE
!DEC$ATTRIBUTES ALIAS: "_XINIT@12":: XInit
!DEC$ENDIF

IMPLICIT NONE

CHARACTER(*)::       szCmdLine
INTEGER,INTENT(IN):: nCmdShow

INTEGER::                  iSt, iErr, iLen, i, nItems, iSize, hKey
TYPE(T_XNSGROUP),POINTER:: tPrev, tGroup
TYPE(X_WINDOW)::           xDesktop
CHARACTER(32)::            sTemp
TYPE(T_NOTIFYICONDATA)::   NID

INCLUDE 'Resource.fd'

!Creation of App & frame window
xDesktop%hWnd = GetDesktopWindow()
iSt = XCreateDialogApp(IDD_XNETSEND, idIcon=IDI_XNETSEND)

hicoSmall = LoadImageA(GetModuleHandle(0), IDI_XNETSEND, 1, 16, 16, 0)
hicoBrowse = LoadImageA(GetModuleHandle(0), IDI_EXPLORE, 1, 16, 16, 0)
hicoAddress = LoadImageA(GetModuleHandle(0), IDI_ADDRESS, 1, 16, 16, 0)

NID%cbSize = SIZEOF(NID)
NID%hWnd = XW_FRAME%hWnd
NID%uID = IDI_XNETSEND
NID%uFlags = NIF_ICON + NIF_MESSAGE + NIF_TIP
NID%uCallbackMessage = WM_ICONCLICKED
NID%hIcon = hicoSmall
NID%szTip = "XNetSend"C
iSt = Shell_NotifyIcon(NIM_ADD, NID)

iSt = XCtlSetSub(XW_FRAME, IDC_BROWSE, XW_FRAME_OnBrowse)
iSt = XCtlSetSub(XW_FRAME, IDC_ADDRESS, XW_FRAME_OnAddressBook)
iSt = XCtlSetSub(XW_FRAME, IDOK, XW_FRAME_OnOKCancel)
iSt = XCtlSetSub(XW_FRAME, IDCANCEL, XW_FRAME_OnOKCancel)

iSt = XCtlSet(XW_FRAME, IDD_XNETSEND, hicoSmall, CTL_ICON)
iSt = XCtlSet(XW_FRAME, IDC_BROWSE, hicoBrowse, CTL_ICON)
iSt = XCtlSet(XW_FRAME, IDC_ADDRESS, hicoAddress, CTL_ICON)

iSt = XSetHandler(XW_FRAME, WM_ICONCLICKED, XW_FRAME_OnNotifyIcon)

iSt = RegCreateKey(HKEY_CURRENT_USER,APP_REGKEY,LOC(hKey))
iSize = 4
iSt = RegQueryValueEx(hKey, "AutoAdd"C, 0, LOC(i), LOC(bAutoAdd), LOC(iSize))
IF (iSt.NE.ERROR_SUCCESS) THEN
      iSt = RegSetValueEx(hKey, "AutoAdd"C, 0, REG_DWORD, LOC(bAutoAdd), 4)
END IF
iSt = RegCloseKey(hKey)


XInit = .TRUE.
iSt = XSetCommand(XW_FRAME, ID_NOTIFY_OPEN, XW_FRAME_OnOpen)
iSt = XSetCommand(XW_FRAME, ID_NOTIFY_CLOSE, XW_FRAME_OnClose)

iSt = GetModuleFileName(0, sBook, LEN(sBook))
sBook = XC2F(sBook)
iLen = LEN_TRIM(sBook)
sBook(iLen-2:iLen) = "txt"

iLen = GetFileAttributes(TRIM(sBook)//CHAR(0))
IF (IAND(iLen,FILE_ATTRIBUTE_READONLY).NE.0) THEN
   iSt=SetFileAttributes(TRIM(sBook)//CHAR(0), FILE_ATTRIBUTE_NORMAL)
   iLen = GetFileAttributes(TRIM(sBook)//CHAR(0))
END IF

OPEN(11, FILE=sBook, IOSTAT=iErr)
nGroups = 0
NULLIFY(tPrev, tGroup, tHead)
DO WHILE(iErr.EQ.0)
   READ(11,"(A32,I5)",IOSTAT=iErr) sTemp, nItems
   IF (iErr.EQ.0) THEN
      ALLOCATE(tGroup)
      ALLOCATE(tGroup%sMember(nItems))
      tGroup%sName = sTemp
      tGroup%nMembers = nItems
      NULLIFY(tGroup%tNext)
      DO i=1, nItems
         READ(11, * ,IOSTAT=iErr) tGroup%sMember(i)
         IF (iErr.NE.0) EXIT
      END DO
      nGroups = nGroups+1
      IF (nGroups.EQ.1) THEN
            tHead => tGroup
      END IF
      IF (ASSOCIATED(tPrev)) THEN
            tPrev%tNext => tGroup
      END IF
      tPrev => tGroup
   END IF
END DO
CLOSE(11)
CALL UpdateList()

END FUNCTION XInit
!=====================================================================
INTEGER FUNCTION XW_FRAME_OnNotifyIcon(xWnd, Msg, wParam, lParam)

INCLUDE 'Resource.fd'

TYPE(X_WINDOW), INTENT(IN)::  xWnd
INTEGER, INTENT(IN)::         Msg, wParam, lParam

INTEGER::            iSt
TYPE(X_MENU)::       xMenu
TYPE(X_MENUITEM)::   xMI
TYPE(T_POINT)::      PT

XW_FRAME_OnNotifyIcon=0
IF (lParam.EQ.WM_LBUTTONDBLCLK) THEN
   CALL XW_FRAME_OnOpen(XW_FRAME, 0,0)
ELSE IF (lParam.EQ.WM_RBUTTONDOWN) THEN
   CALL XDestroyMenu(xMenu)
   xMenu = XCreateMenu(.TRUE.)
   xMI = X_MENUITEM(MF_STRING, MF_ENABLED, ID_NOTIFY_OPEN, "Open"C)
   iSt = XInsertMenuItem(xMenu, xMI, (/XMENU_APPEND/))
   xMI = X_MENUITEM(MF_STRING, MF_ENABLED, ID_NOTIFY_CLOSE, "Close"C)
   iSt = XInsertMenuItem(xMenu, xMI, (/XMENU_APPEND/))
   iSt = GetCursorPos(PT)
   iSt = SetForegroundWindow(XW_FRAME%hWnd)
   iSt = XContextMenu(xMenu, XW_FRAME, PT%x, PT%y)
   iSt = XSendMessage(XW_FRAME, WM_NULL, 0, 0)
END IF

END FUNCTION XW_FRAME_OnNotifyIcon
!=====================================================================
SUBROUTINE XW_FRAME_OnOpen(xWnd, ID, nCode)

INCLUDE "Resource.fd"

TYPE(X_WINDOW)::      xWnd
INTEGER, INTENT(IN):: ID, nCode

INTEGER::            iSt, nTotal, i, iErr
LOGICAL::            bFound

iSt = XCtlSet(xWnd, IDC_EDIT_MSG, MAX_TEXTLEN-2, CTL_RANGEMAX)

iSt = XShowWindow(xWnd, SW_SHOW)
iSt = SetForegroundWindow(xWnd%hWnd)

END SUBROUTINE XW_FRAME_OnOpen
!=====================================================================
SUBROUTINE XW_FRAME_OnClose(xWnd, ID, nCode)

IMPLICIT NONE

INCLUDE "Resource.fd"

TYPE(X_WINDOW)::      xWnd
INTEGER, INTENT(IN):: ID, nCode

INTEGER::            iSt
TYPE(T_NOTIFYICONDATA)::   NID

NID%cbSize = SIZEOF(NID)
NID%hWnd = xWnd%hWnd
NID%uID = IDI_XNETSEND
NID%uFlags = 0

iSt = Shell_NotifyIcon(NIM_DELETE, NID)
iSt = DestroyWindow(xWnd%hWnd)

END SUBROUTINE XW_FRAME_OnClose
!===========================================================
SUBROUTINE XW_FRAME_OnOKCancel(xDlg,ID, nCode)

INCLUDE "Resource.fd"

TYPE(X_WINDOW)::      xDlg
INTEGER, INTENT(IN):: ID, nCode

INTEGER::            iSt, nTotal, i, iErr
CHARACTER(1000)::    sText
CHARACTER(32)::      sServer
CHARACTER, POINTER:: NULL_CHAR

IF (ID.EQ.IDOK) THEN
   iSt = XCtlGet(xDlg, IDC_COMBO_TO, sServer, CTL_STATE)
   iSt = INDEX(sServer, "[")
   IF (iSt.NE.0) sServer=sServer(1:iSt-1)
   iSt = XCtlGet(xDlg, IDC_EDIT_MSG, sText, CTL_STATE)

   CALL XW_FRAME_NetSend(TRIM(sServer), TRIM(sText))

END IF
iSt=XShowWindow(xDlg,SW_HIDE)

END SUBROUTINE XW_FRAME_OnOKCancel

!=====================================================================
SUBROUTINE XW_FRAME_NetSend(sServer, sText)

CHARACTER(*), INTENT(IN)::    sServer
CHARACTER(*), INTENT(IN)::    sText

INTEGER::                     iSt, nTotal, iErr, i
INTEGER,ALLOCATABLE::         iError(:)
!INTEGER(2)::                  uniText(MAX_TEXTLEN), uniServer(32)
CHARACTER(256)::              szErr
TYPE(T_XNSGROUP), POINTER::   tGr, tPrev
CHARACTER, POINTER:: NULL_CHAR

tGr => GetGroupByName(sServer, tPrev)
IF (ASSOCIATED(tGr)) THEN
   IF (tGr%nMembers.EQ.0) THEN
      iErr = NetMessageBufferSend(0, XUnicode(sServer), 0, XUnicode(sText), 2*LEN_TRIM(sText)+2)
      IF (iErr.NE.0) THEN
            iSt = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, iErr, 0, szErr, LEN(szErr), 0)
            iErr = XMessageBox("Send failed:"//CHAR(13)//szErr, "XNetSend", MB_ICONSTOP.OR.MB_OK)
      END IF

   ELSE
      ALLOCATE(iError(tGr%nMembers))
      DO i = 1, tGr%nMembers
         iError(i) = NetMessageBufferSend(0, XUnicode(sServer), 0, XUnicode(sText), 2*LEN_TRIM(sText)+2)
      END DO
      IF (ANY(iError.NE.0)) THEN
         szErr = "Send failed to the following recipients: "//CHAR(13)
         DO i=1,tGr%nMembers
            IF (iError(i).NE.0) szErr = TRIM(szErr)//TRIM(tGr%sMember(i))//','
         END DO
         iErr = XMessageBox(TRIM(szErr), "XNetSend", MB_ICONSTOP.OR.MB_OK)
      END IF
      DEALLOCATE(iError)
   END IF
ELSE
   iErr = NetMessageBufferSend(0, XUnicode(sServer), 0, XUnicode(sText), 2*LEN_TRIM(sText)+2)
   IF (iErr.NE.0) THEN
         iSt = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, iErr, 0, szErr, LEN(szErr), 0)
         iErr = XMessageBox("Send failed:"//CHAR(13)//szErr, "XNetSend", MB_ICONSTOP.OR.MB_OK)
   END IF
   IF (bAutoAdd) THEN
      ALLOCATE(tGr)
      IF (ASSOCIATED(tPrev)) THEN
            tPrev%tNext => tGr
      ELSE
            tHead => tGr
      END IF
      NULLIFY(tGr%tNext)
      NULLIFY(tGr%sMember)
      tGr%nMembers = 0
      tGr%sName = sServer
      CALL UpdateList()
      CALL SaveAddressBook()
   END IF
END IF

END SUBROUTINE XW_FRAME_NetSend
!=====================================================================
SUBROUTINE XW_FRAME_OnBrowse(xDlg, ID, nCode)

TYPE(X_WINDOW)::  xDlg
INTEGER, INTENT(IN)::   ID, nCode

INCLUDE "Resource.fd"

INTEGER, PARAMETER:: CSIDL_NETWORK                   = #0012
INTEGER, PARAMETER:: BIF_BROWSEFORCOMPUTER  = #1000  ! Browsing for Computers.
INTEGER::            iSt
CHARACTER(32)::      sComputer

iSt = XBrowse(xDlg, sComputer, BIF_BROWSEFORCOMPUTER, CSIDL_NETWORK)
iSt = XCtlSet(xDlg, IDC_COMBO_TO, sComputer, CTL_STATE)

END SUBROUTINE XW_FRAME_OnBrowse

!=====================================================================
SUBROUTINE XW_FRAME_OnAddressBook(xDlg, ID, nCode)

TYPE(X_WINDOW)::              xDlg, xAddDlg
INTEGER, INTENT(IN)::         ID, nCode

CHARACTER(32)::               sName
INTEGER::                     iSt, hKey
LOGICAL::                     bAdd

INCLUDE "Resource.fd"

iSt = XLoadDialog(IDD_ADDRESS, xAddDlg, xDlg)
iSt = XCtlSet(xAddDlg, IDD_ADDRESS, hicoAddress, CTL_ICON)
iSt = XCtlSet(xAddDlg, IDC_LIST_ADDRESS, 1, CTL_SELECTION)
iSt = XCtlSet(xAddDlg, IDC_CHECK_AUTO, bAutoAdd, CTL_STATE)

iSt = XCtlSetSub(xAddDlg, IDC_LIST_ADDRESS, XAddDlg_OnAddressChange, CTL_SELCHANGE)
iSt = XCtlSetSub(xAddDlg, IDC_LIST_ADDRESS, XAddDlg_OnAddressChange, CTL_DBLCLICK)
iSt = XCtlSetSub(xAddDlg, IDC_ADD, XAddDlg_OnGroup)
iSt = XCtlSetSub(xAddDlg, IDC_EDIT, XAddDlg_OnGroup)
iSt = XCtlSetSub(xAddDlg, IDC_REMOVE, XAddDlg_OnGroup)
CALL UpdateList(1, xAddDlg)

IF (XModalDialog(xAddDlg).EQ.IDOK) THEN
   iSt = XCtlGet(xAddDlg, IDC_CHECK_AUTO, bAdd, CTL_STATE)
   IF (bAdd.NEQV.bAutoAdd) THEN
      bAutoAdd = bAdd
      iSt = RegCreateKey(HKEY_CURRENT_USER,APP_REGKEY,LOC(hKey))
      iSt = RegSetValueEx(hKey, "AutoAdd"C, 0, REG_DWORD, LOC(bAutoAdd), 4)
      iSt = RegCloseKey(hKey)
   END IF
   iSt = XCtlGet(xAddDlg, IDC_EDIT_NAME, sName, CTL_STATE)
   iSt = XCtlSet(XW_FRAME, IDC_COMBO_TO, sName, CTL_STATE)
END IF

CALL XDestroyDialog(xAddDlg)

END SUBROUTINE XW_FRAME_OnAddressBook

!=====================================================================
SUBROUTINE UpdateList(iSel, xAddDlg)

TYPE(X_WINDOW), OPTIONAL::  xAddDlg
INTEGER, OPTIONAL::         iSel

INTEGER::                     iSt, i, iSel2
TYPE(T_XNSGROUP), POINTER::   tGr

INCLUDE "Resource.fd"

IF (PRESENT(xAddDlg)) iSt = XCtlSet(xAddDlg, IDC_LIST_ADDRESS, 0, CTL_NUMITEMS)
iSt = XCtlGet(XW_FRAME, IDC_COMBO_TO, iSel2, CTL_SELECTION)
iSt = XCtlSet(XW_FRAME, IDC_COMBO_TO, 0, CTL_NUMITEMS)
tGr => tHead
i = 1
DO WHILE (ASSOCIATED(tGr))
      IF (tGr%nMembers.GT.0) THEN
            IF (PRESENT(xAddDlg)) iSt = XCtlSet(xAddDlg, IDC_LIST_ADDRESS, TRIM(tGr%sName)//" [Group]", 0)
            iSt = XCtlSet(XW_FRAME, IDC_COMBO_TO, TRIM(tGr%sName)//" [Group]", 0)
      ELSE
            IF (PRESENT(xAddDlg)) iSt = XCtlSet(xAddDlg, IDC_LIST_ADDRESS, tGr%sName, 0)
            iSt = XCtlSet(XW_FRAME, IDC_COMBO_TO, tGr%sName, 0)
      END IF
      tGr => tGr%tNext
      i = i+1
END DO

IF (PRESENT(xAddDlg)) THEN
      IF (.NOT.XCtlSet(xAddDlg, IDC_LIST_ADDRESS, iSel, CTL_SELECTION)) THEN
         iSt = XCtlSet(xAddDlg, IDC_LIST_ADDRESS, 1, CTL_SELECTION)
      END IF
      CALL XAddDlg_OnAddressChange(xAddDlg, IDC_LIST_ADDRESS, CTL_SELCHANGE)
END IF
iSt = XCtlSet(XW_FRAME, IDC_COMBO_TO, iSel2, CTL_SELECTION)

END SUBROUTINE UpdateList

!=====================================================================
SUBROUTINE XAddDlg_OnAddressChange(xAddDlg, ID, nCode)

TYPE(X_WINDOW)::        xAddDlg
INTEGER, INTENT(IN)::   ID, nCode

INTEGER::               iSt, iLen
CHARACTER(32)::         sAddr

INCLUDE "Resource.fd"

iSt = XCtlGet(xAddDlg, ID, sAddr, CTL_SELECTION)
!iLen = INDEX(sAddr, "[")
!IF (iLen.NE.0) sAddr=sAddr(1:iLen-1)
iSt = XCtlSet(xAddDlg, IDC_EDIT_NAME, sAddr, CTL_STATE)
IF (nCode.EQ.CTL_DBLCLICK) CALL XEndDialog(xAddDlg, IDOK)

END SUBROUTINE XAddDlg_OnAddressChange

!=====================================================================
SUBROUTINE XAddDlg_OnGroup(xAddDlg, ID, nCode)

TYPE(X_WINDOW)::        xAddDlg, xGrDlg
INTEGER, INTENT(IN)::   ID, nCode

TYPE(T_XNSGROUP), POINTER::  tGr, locGr, tPrev
CHARACTER(32)::         sAddr
INTEGER::               iSt, i, iSel, iLen

INCLUDE "Resource.fd"

iSt = XCtlGet(xAddDlg, IDC_LIST_ADDRESS, iSel, CTL_SELECTION)
iSt = XCtlGet(xAddDlg, IDC_LIST_ADDRESS, sAddr, CTL_SELECTION)
iLen = INDEX(sAddr, "[")
IF (iLen.NE.0) sAddr = sAddr(1:iLen-1)
SELECT CASE(ID)
CASE(IDC_REMOVE)
   tGr => GetGroupByName(sAddr, tPrev)
   IF (ASSOCIATED(tPrev)) tPrev%tNext => tGr%tNext
   IF (ASSOCIATED(tHead,tGr)) tHead => tGr%tNext
   DEALLOCATE(tGr%sMember, STAT=iSt)
   DEALLOCATE(tGr)
   CALL UpdateList(iSel, xAddDlg)
   CALL SaveAddressBook()
CASE(IDC_ADD, IDC_EDIT)
   iSt = XLoadDialog(IDD_GROUP, xGrDlg, xAddDlg)

   IF (ID.EQ.IDC_EDIT) THEN
         iSt = XCtlSet(xGrDlg, IDC_EDIT_NAME, sAddr, CTL_STATE)
         tGr => GetGroupByName(sAddr, tPrev)
         CALL XGrDlg_UpdateCombo(xGrDlg, tGr)
   ELSE
         ALLOCATE(locGr)
         NULLIFY(locGr%tNext)
         NULLIFY(locGr%sMember)
         iSt = XCtlSet(xGrDlg, IDC_EDIT_NAME, "New group", CTL_STATE)
         tGr => GetGroupByName("@@", tPrev)
         tGr => locGr

         iSt = XCtlSetSub(xGrDlg, IDOK, XGrDlg_OnOK)
   END IF
   
   iSt = XCtlSetSub(xGrDlg, IDC_ADD, XGrDlg_OnMemberChange)
   iSt = XCtlSetSub(xGrDlg, IDC_REMOVE, XGrDlg_OnMemberChange)

   IF (XModalDialog(xGrDlg).EQ.IDOK) THEN
         iSt = XCtlGet(xGrDlg, IDC_EDIT_NAME, sAddr, CTL_STATE)
         tGr%sName = sAddr
         iSt = XCtlGet(xGrDlg, IDC_COMBO_MEMBERS, tGr%nMembers, CTL_NUMITEMS)
         DEALLOCATE(tGr%sMember, STAT=iSt)
         ALLOCATE(tGr%sMember(tGr%nMembers))
         DO i=1,tGr%nMembers
               iSt = XCtlGet(xGrDlg, IDC_COMBO_MEMBERS, tGr%sMember(i), i)
         END DO
         IF (ASSOCIATED(tPrev)) THEN
               tPrev%tNext => tGr
         ELSE
               tHead => tGr
         END IF
         CALL UpdateList(iSel, xAddDlg)
         CALL SaveAddressBook()
   ELSE 
         IF (ASSOCIATED(locGr)) DEALLOCATE(locGr)
   END IF
END SELECT

END SUBROUTINE XAddDlg_OnGroup

!=====================================================================
SUBROUTINE XGrDlg_OnOK(xGrDlg, ID, nCode)

TYPE(X_WINDOW)::        xGrDlg
INTEGER, INTENT(IN)::   ID, nCode

INTEGER::               iSt
CHARACTER(32)::         sName

INCLUDE "Resource.fd"

iSt = XCtlGet(xGrDlg, IDC_EDIT_NAME, sName, CTL_STATE)
IF (ASSOCIATED(GetGroupByName(sName))) THEN
    iSt = XMessageBox("Item with name "//TRIM(sName)//" already exists.", "Error", MB_OK.OR.MB_ICONSTOP, xGrDlg)
ELSE
    CALL XEndDialog(xGrDlg, IDOK)
END IF

END SUBROUTINE XGrDlg_OnOK

!=====================================================================
SUBROUTINE xGrDlg_OnMemberChange(xGrDlg, ID, nCode)

TYPE(X_WINDOW)::        xGrDlg
INTEGER, INTENT(IN)::   ID, nCode

INTEGER::               iSt, iSel, Index
CHARACTER(32)::         sName

INCLUDE "Resource.fd"

SELECT CASE(ID)
CASE(IDC_REMOVE)
   IF (XCtlGet(xGrDlg, IDC_COMBO_MEMBERS, iSel, CTL_SELECTION)) THEN
      iSt = SendMessage(GetDlgItem(xGrDlg%hWnd, IDC_COMBO_MEMBERS), CB_DELETESTRING, iSel-1, 0)
   END IF
CASE(IDC_ADD)
   IF (XCtlGet(xGrDlg, IDC_COMBO_MEMBERS, sName, CTL_SELECTION)) THEN
      sName=TRIM(sName)//CHAR(0)
      Index = SendMessage(GetDlgItem(xGrDlg%hWnd, IDC_COMBO_MEMBERS), CB_FINDSTRINGEXACT, -1, LOC(sName))
      IF (Index.EQ.CB_ERR .AND. LEN_TRIM(sName).GT.1) THEN
         sName=XC2F(sName)
         iSt = XCtlSet(xGrDlg, IDC_COMBO_MEMBERS, sName, 0)
      ELSE
         !Warn
      END IF
   END IF
END SELECT

END SUBROUTINE xGrDlg_OnMemberChange
!=====================================================================
SUBROUTINE XGrDlg_UpdateCombo(xGrDlg, tGr)

TYPE(X_WINDOW)::              xGrDlg
TYPE(T_XNSGROUP), POINTER::   tGr

INTEGER::                     i, iSt

INCLUDE "Resource.fd"

iSt = XCtlSet(xGrDlg, IDC_COMBO_MEMBERS, 0, CTL_NUMITEMS)
DO i = 1,tGr%nMembers
   iSt = XCtlSet(xGrDlg, IDC_COMBO_MEMBERS, TRIM(tGr%sMember(i)), 0)
END DO

END SUBROUTINE XGrDlg_UpdateCombo

!=====================================================================
FUNCTION GetGroupByName(sName, tPrev) RESULT(tGr)

TYPE(T_XNSGROUP), POINTER::      tGr
CHARACTER(*)::                   sName
TYPE(T_XNSGROUP), POINTER, OPTIONAL::      tPrev

tGr => tHead
IF (PRESENT(tPrev)) NULLIFY(tPrev)
DO WHILE (ASSOCIATED(tGr))
   IF (TOLOWER(tGr%sName) .EQ. TOLOWER(sName)) THEN
      RETURN
   END IF
   IF (PRESENT(tPrev)) tPrev => tGr
   tGr => tGr%tNext
END DO

END FUNCTION GetGroupByName
!=====================================================================

SUBROUTINE SaveAddressBook()

TYPE(T_XNSGROUP), POINTER::   tGr

INTEGER::                     i, iErr

OPEN(11, FILE=sBook, IOSTAT=iErr)
IF (iErr.EQ.0) THEN
   tGr => tHead
   DO WHILE (ASSOCIATED(tGr))
      WRITE(11, "(A,I3)") tGr%sName, tGr%nMembers
      DO i=1,tGr%nMembers
         WRITE(11, "(1x,A)", IOSTAT=iErr) TRIM(tGr%sMember(i))
         IF (iErr.NE.0) THEN
            iErr = XMessageBox("Cannot save file:"//CHAR(13)//TRIM(sBook), "Error"C, MB_ICONSTOP.OR.MB_OK)
            RETURN
         END IF            
      END DO
      tGr => tGr%tNext
   END DO
END IF
CLOSE(11)

END SUBROUTINE SaveAddressBook
!=====================================================================
END MODULE XNetSendM


